@@ -118,7 +118,11 @@ class CreateDevice:
             self.ptzfocus_url = '%s/ISAPI/System/Video/inputs/channels/1/focus' % self._base
             # self.ir_url = '%s/ISAPI/Thermal/channels/2/thermometry/pixelToPixelParam/capabilities' % self._base
             self.ir_url = '%s/ISAPI/Thermal/channels/2/thermometry/jpegPicWithAppendData?format=json' % self._base
-#            self._xml_namespace = "{http://www.hikvision.com/ver20/XMLSchema}"
+            self.capabilities_url = '%s/ISAPI/PTZCtrl/channels/1/absoluteEx/capabilities' % self._base
+            self.ptz_absoluteEx_url = '%s/ISAPI/PTZCtrl/channels/1/absoluteEx' % self._base
+            self.onepushfocus_url = '%s/ISAPI/PTZCtrl/channels/1/onepushfoucs/start' % self._base
+            self.absoluteStatus_url = '%s/ISAPI/PTZCtrl/channels/1/status' % self._base
+
         else:
             self.motion_url = '%s/MotionDetection/1' % self._base
             self.deviceinfo_url = '%s/System/deviceInfo' % self._base
@@ -132,6 +136,106 @@ class CreateDevice:
         self._xml_namespace = ""
 
     
+    def get_absolutestatus(self, element_to_query=None):
+        response = requests.get(
+            self.ptz_absoluteEx_url,
+            auth=self._auth_fn(self._username, self._password),
+            headers={'Content-Type': 'application/json'}
+            )
+        if response.status_code != 200:
+            return None
+
+        if element_to_query is None:
+            # 返回的是json格式的数据，返回的是一个字典
+            return response.text
+        try:
+            tree = tree_no_ns_from_string(response.text)
+
+            element_to_query = './/%s%s' % (
+                self._xml_namespace, element_to_query)
+            result = tree.findall(element_to_query)
+            if len(result) > 0:
+                
+
+                return result[0].text.strip()
+            
+
+        except AttributeError as attib_err:
+            
+            return None
+        return None
+    
+    def set_ptzabsoluteEx(self,elevation,azimuth,absoluteZoom ,focus,focalLen,horizontalSpeed=10.00,verticalSpeed=10.00,zoomType='focalLen',element_to_query=None):
+        data_xml = '<?xml version="1.0" encoding="UTF-8"?>' +\
+                    '<PTZAbsoluteEx version="2.0" xmlns="http://www.isapi.org/ver20/XMLSchema">' +\
+                    '<elevation>'+ elevation +'</elevation>'+\
+                    '<azimuth>'+ azimuth +'</azimuth>'+\
+                    '<absoluteZoom>'+ absoluteZoom +'</absoluteZoom>'+\
+                    '<focus>'+ focus +'</focus>'+\
+                    '<focalLen>'+ focalLen +'</focalLen>'+\
+                    '<horizontalSpeed>'+ horizontalSpeed +'</horizontalSpeed>'+\
+                    '<verticalSpeed>'+ verticalSpeed +'</verticalSpeed>'+\
+                    '<zoomType>'+ zoomType +'</zoomType>'+\
+                    '</PTZAbsoluteEx>'
+        # 转换为UTF-8编码
+        data_xml = data_xml.encode('utf-8')
+        print(data_xml)
+        response = requests.put(
+            self.ptz_absoluteEx_url,
+            auth=self._auth_fn(self._username, self._password),
+            data=data_xml,
+            )
+        if response.status_code != 200:
+            print('http code:',response.status_code)
+            print('response:',response.text)
+            # log_response_errors(response)
+            return None
+
+        if element_to_query is None:
+            print(response.status_code)
+            print(response.text)
+            return response.text
+        try:
+            tree = tree_no_ns_from_string(response.text)
+
+            element_to_query = './/%s%s' % (
+                self._xml_namespace, element_to_query)
+            result = tree.findall(element_to_query)
+            if len(result) > 0:
+                return result[0].text.strip()
+        except AttributeError as attib_err:
+            return None
+        return None
+    
+    def set_onepushfocus(self,element_to_query=None):
+        response = requests.put(
+            self.onepushfocus_url,
+            auth=self._auth_fn(self._username, self._password),
+            headers={'Content-Type': 'application/json'}
+            )
+        if response.status_code != 200:
+            return response.text
+
+        if element_to_query is None:
+            # 返回的是json格式的数据，返回的是一个字典
+            return response.text
+        try:
+            tree = tree_no_ns_from_string(response.text)
+
+            element_to_query = './/%s%s' % (
+                self._xml_namespace, element_to_query)
+            result = tree.findall(element_to_query)
+            if len(result) > 0:
+                
+
+                return result[0].text.strip()
+            
+
+        except AttributeError as attib_err:
+            
+            return None
+        return None
+    
     def ir_test(self, element_to_query=None):
         response = requests.get(
             self.ir_url,
@@ -280,7 +384,7 @@ class CreateDevice:
             data=data_xml,
             )
         if response.status_code != 200:
-            return None
+            return response.text
 
         if element_to_query is None:
             return response.text
@@ -315,8 +419,7 @@ class CreateDevice:
             data=data_xml,
             )
         if response.status_code != 200:
-            print(response)
-            # log_response_errors(response)
+            log_response_errors(response)
             return None
 
         if element_to_query is None:
@@ -334,8 +437,7 @@ class CreateDevice:
             return None
         return None
         
-        
-        
+           
     def set_ptz(self, element_to_query=None, up_down=0, left_right=0, zoom=0):
         pan =left_right * 15
         tilt = -up_down * 15
